@@ -4,10 +4,6 @@
 #include <LightGBM/meta.h>
 #include <LightGBM/config.h>
 
-#ifdef USE_PROTO
-#include "model.pb.h"
-#endif // USE_PROTO
-
 #include <vector>
 #include <string>
 #include <map>
@@ -53,6 +49,8 @@ public:
 
   virtual void ResetConfig(const BoostingConfig* config) = 0;
 
+
+
   /*!
   * \brief Add a validation data
   * \param valid_data Validation data
@@ -63,6 +61,11 @@ public:
 
   virtual void Train(int snapshot_freq, const std::string& model_output_path) = 0;
 
+  /*!
+  * \brief Update the tree output by new training data
+  */
+  virtual void RefitTree(const std::vector<std::vector<int>>& tree_leaf_prediction) = 0;
+
   /*!
   * \brief Training logic
   * \param gradients nullptr for using default objective, otherwise use self-defined boosting
@@ -198,26 +201,11 @@ public:
 
   /*!
   * \brief Restore from a serialized string
-  * \param model_str The string of model
+  * \param buffer The content of model
+  * \param len The length of buffer
   * \return true if succeeded
   */
-  virtual bool LoadModelFromString(const std::string& model_str) = 0;
-
-  #ifdef USE_PROTO
-  /*!
-  * \brief Save model with protobuf
-  * \param num_iterations Number of model that want to save, -1 means save all
-  * \param filename Filename that want to save to
-  */
-  virtual void SaveModelToProto(int num_iteration, const char* filename) const = 0;
-
-  /*!
-  * \brief Restore from a serialized protobuf file
-  * \param filename Filename that want to restore from
-  * \return true if succeeded
-  */
-  virtual bool LoadModelFromProto(const char* filename) = 0;
-  #endif // USE_PROTO
+  virtual bool LoadModelFromString(const char* buffer, size_t len) = 0;
 
   /*!
   * \brief Calculate feature importances
@@ -283,7 +271,7 @@ public:
   /*! \brief Disable copy */
   Boosting(const Boosting&) = delete;
 
-  static bool LoadFileToBoosting(Boosting* boosting, const std::string& format, const char* filename);
+  static bool LoadFileToBoosting(Boosting* boosting, const char* filename);
 
   /*!
   * \brief Create boosting object
@@ -293,7 +281,7 @@ public:
   * \param filename name of model file, if existing will continue to train from this model
   * \return The boosting object
   */
-  static Boosting* CreateBoosting(const std::string& type, const std::string& format, const char* filename);
+  static Boosting* CreateBoosting(const std::string& type, const char* filename);
 
 };
 
