@@ -87,7 +87,7 @@ public:
 
 /*! \brief Types of tasks */
 enum TaskType {
-  kTrain, kPredict, kConvertModel
+  kTrain, kPredict, kConvertModel, KRefitTree
 };
 
 /*! \brief Config for input and output files */
@@ -105,7 +105,6 @@ public:
   std::string output_result = "LightGBM_predict_result.txt";
   std::string convert_model = "gbdt_prediction.cpp";
   std::string input_model = "";
-  std::string model_format = "text";
   int verbosity = 1;
   int num_iteration_predict = -1;
   bool is_pre_partition = false;
@@ -403,6 +402,7 @@ struct ParameterAlias {
       { "num_trees", "num_iterations" },
       { "num_rounds", "num_iterations" },
       { "num_boost_round", "num_iterations" },
+      { "n_estimators", "num_iterations"},
       { "sub_row", "bagging_fraction" },
       { "subsample", "bagging_fraction" },
       { "subsample_freq", "bagging_freq" },
@@ -449,7 +449,7 @@ struct ParameterAlias {
     const std::unordered_set<std::string> parameter_set({
       "config", "config_file", "task", "device",
       "num_threads", "seed", "boosting_type", "objective", "data",
-      "output_model", "input_model", "output_result", "model_format", "valid_data",
+      "output_model", "input_model", "output_result", "valid_data",
       "is_enable_sparse", "is_pre_partition", "is_training_metric",
       "ndcg_eval_at", "min_data_in_leaf", "min_sum_hessian_in_leaf",
       "num_leaves", "feature_fraction", "num_iterations",
