@@ -17,11 +17,15 @@
 #include <type_traits>
 #include <iomanip>
 
+#ifdef _MSC_VER
+#include "intrin.h"
+#endif
+
 namespace LightGBM {
 
 namespace Common {
 
-inline char tolower(char in) {
+inline static char tolower(char in) {
   if (in <= 'Z' && in >= 'A')
     return in - ('Z' - 'z');
   return in;
@@ -128,18 +132,10 @@ inline static std::vector<std::string> Split(const char* c_str, const char* deli
   return ret;
 }
 
-inline static std::string FindFromLines(const std::vector<std::string>& lines, const char* key_word) {
-  for (auto& line : lines) {
-    size_t find_pos = line.find(key_word);
-    if (find_pos != std::string::npos) {
-      return line;
-    }
-  }
-  return "";
-}
-
-inline static const char* Atoi(const char* p, int* out) {
-  int sign, value;
+template<typename T>
+inline static const char* Atoi(const char* p, T* out) {
+  int sign;
+  T value;
   while (*p == ' ') {
     ++p;
   }
@@ -153,14 +149,14 @@ inline static const char* Atoi(const char* p, int* out) {
   for (value = 0; *p >= '0' && *p <= '9'; ++p) {
     value = value * 10 + (*p - '0');
   }
-  *out = sign * value;
+  *out = static_cast<T>(sign * value);
   while (*p == ' ') {
     ++p;
   }
   return p;
 }
 
-template<class T>
+template<typename T>
 inline static double Pow(T base, int power) {
   if (power < 0) {
     return 1.0 / Pow(base, -power);
@@ -267,7 +263,7 @@ inline static const char* Atof(const char* p, double* out) {
   return p;
 }
 
-inline bool AtoiAndCheck(const char* p, int* out) {
+inline static bool AtoiAndCheck(const char* p, int* out) {
   const char* after = Atoi(p, out);
   if (*after != '\0') {
     return false;
@@ -275,7 +271,7 @@ inline bool AtoiAndCheck(const char* p, int* out) {
   return true;
 }
 
-inline bool AtofAndCheck(const char* p, double* out) {
+inline static bool AtofAndCheck(const char* p, double* out) {
   const char* after = Atof(p, out);
   if (*after != '\0') {
     return false;
@@ -283,6 +279,97 @@ inline bool AtofAndCheck(const char* p, double* out) {
   return true;
 }
 
+inline static unsigned CountDecimalDigit32(uint32_t n) {
+#if defined(_MSC_VER) || defined(__GNUC__)
+  static const uint32_t powers_of_10[] = {
+    0,
+    10,
+    100,
+    1000,
+    10000,
+    100000,
+    1000000,
+    10000000,
+    100000000,
+    1000000000
+  };
+#ifdef _MSC_VER
+  unsigned long i = 0;
+  _BitScanReverse(&i, n | 1);
+  uint32_t t = (i + 1) * 1233 >> 12;
+#elif __GNUC__
+  uint32_t t = (32 - __builtin_clz(n | 1)) * 1233 >> 12;
+#endif
+  return t - (n < powers_of_10[t]) + 1;
+#else
+  if (n < 10) return 1;
+  if (n < 100) return 2;
+  if (n < 1000) return 3;
+  if (n < 10000) return 4;
+  if (n < 100000) return 5;
+  if (n < 1000000) return 6;
+  if (n < 10000000) return 7;
+  if (n < 100000000) return 8;
+  if (n < 1000000000) return 9;
+  return 10;
+#endif
+}
+
+inline static void Uint32ToStr(uint32_t value, char* buffer) {
+  const char kDigitsLut[200] = {
+    '0','0','0','1','0','2','0','3','0','4','0','5','0','6','0','7','0','8','0','9',
+    '1','0','1','1','1','2','1','3','1','4','1','5','1','6','1','7','1','8','1','9',
+    '2','0','2','1','2','2','2','3','2','4','2','5','2','6','2','7','2','8','2','9',
+    '3','0','3','1','3','2','3','3','3','4','3','5','3','6','3','7','3','8','3','9',
+    '4','0','4','1','4','2','4','3','4','4','4','5','4','6','4','7','4','8','4','9',
+    '5','0','5','1','5','2','5','3','5','4','5','5','5','6','5','7','5','8','5','9',
+    '6','0','6','1','6','2','6','3','6','4','6','5','6','6','6','7','6','8','6','9',
+    '7','0','7','1','7','2','7','3','7','4','7','5','7','6','7','7','7','8','7','9',
+    '8','0','8','1','8','2','8','3','8','4','8','5','8','6','8','7','8','8','8','9',
+    '9','0','9','1','9','2','9','3','9','4','9','5','9','6','9','7','9','8','9','9'
+  };
+  unsigned digit = CountDecimalDigit32(value);
+  buffer += digit;
+  *buffer = '\0';
+
+  while (value >= 100) {
+    const unsigned i = (value % 100) << 1;
+    value /= 100;
+    *--buffer = kDigitsLut[i + 1];
+    *--buffer = kDigitsLut[i];
+  }
+
+  if (value < 10) {
+    *--buffer = char(value) + '0';
+  }
+  else {
+    const unsigned i = value << 1;
+    *--buffer = kDigitsLut[i + 1];
+    *--buffer = kDigitsLut[i];
+  }
+}
+
+inline static void Int32ToStr(int32_t value, char* buffer) {
+  uint32_t u = static_cast<uint32_t>(value);
+  if (value < 0) {
+    *buffer++ = '-';
+    u = ~u + 1;
+  }
+  Uint32ToStr(u, buffer);
+}
+
+inline static void DoubleToStr(double value, char* buffer, size_t 
+                               #ifdef _MSC_VER
+                               buffer_len
+                               #endif
+) {
+  #ifdef _MSC_VER
+  sprintf_s(buffer, buffer_len, "%.17g", value);
+  #else
+  sprintf(buffer, "%.17g", value);
+  #endif
+}
+
 inline static const char* SkipSpaceAndTab(const char* p) {
   while (*p == ' ' || *p == '\t') {
     ++p;
@@ -299,39 +386,72 @@ inline static const char* SkipReturn(const char* p) {
 
 template<typename T, typename T2>
 inline static std::vector<T2> ArrayCast(const std::vector<T>& arr) {
-  std::vector<T2> ret;
+  std::vector<T2> ret(arr.size());
   for (size_t i = 0; i < arr.size(); ++i) {
-    ret.push_back(static_cast<T2>(arr[i]));
+    ret[i] = static_cast<T2>(arr[i]);
   }
   return ret;
 }
 
+template<typename T, bool is_float, bool is_unsign>
+struct __TToStringHelperFast {
+  void operator()(T value, char* buffer, size_t ) const {
+    Int32ToStr(value, buffer);
+  }
+};
+
 template<typename T>
-inline static std::string ArrayToString(const std::vector<T>& arr, char delimiter) {
-  if (arr.empty()) {
+struct __TToStringHelperFast<T, true, false> {
+  void operator()(T value, char* buffer, size_t 
+                  #ifdef _MSC_VER
+                  buf_len
+                  #endif
+                  ) const {
+    #ifdef _MSC_VER
+    sprintf_s(buffer, buf_len, "%g", value);
+    #else
+    sprintf(buffer, "%g", value);
+    #endif
+  }
+};
+
+template<typename T>
+struct __TToStringHelperFast<T, false, true> {
+  void operator()(T value, char* buffer, size_t ) const {
+    Uint32ToStr(value, buffer);
+  }
+};
+
+template<typename T>
+inline static std::string ArrayToStringFast(const std::vector<T>& arr, size_t n) {
+  if (arr.empty() || n == 0) {
     return std::string("");
   }
+  __TToStringHelperFast<T, std::is_floating_point<T>::value, std::is_unsigned<T>::value> helper;
+  const size_t buf_len = 16;
+  std::vector<char> buffer(buf_len);
   std::stringstream str_buf;
-  str_buf << std::setprecision(std::numeric_limits<double>::digits10 + 2);
-  str_buf << arr[0];
-  for (size_t i = 1; i < arr.size(); ++i) {
-    str_buf << delimiter;
-    str_buf << arr[i];
+  helper(arr[0], buffer.data(), buf_len);
+  str_buf << buffer.data();
+  for (size_t i = 1; i < std::min(n, arr.size()); ++i) {
+    helper(arr[i], buffer.data(), buf_len);
+    str_buf << ' ' << buffer.data();
   }
   return str_buf.str();
 }
 
-template<typename T>
-inline static std::string ArrayToString(const std::vector<T>& arr, size_t n, char delimiter) {
+inline static std::string ArrayToString(const std::vector<double>& arr, size_t n) {
   if (arr.empty() || n == 0) {
     return std::string("");
   }
+  const size_t buf_len = 32;
+  std::vector<char> buffer(buf_len);
   std::stringstream str_buf;
-  str_buf << std::setprecision(std::numeric_limits<double>::digits10 + 2);
-  str_buf << arr[0];
+  DoubleToStr(arr[0], buffer.data(), buf_len);
+  str_buf << buffer.data();
   for (size_t i = 1; i < std::min(n, arr.size()); ++i) {
-    str_buf << delimiter;
-    str_buf << arr[i];
+    DoubleToStr(arr[i], buffer.data(), buf_len);
+    str_buf << ' ' << buffer.data();
   }
   return str_buf.str();
 }
@@ -339,7 +459,9 @@ inline static std::string ArrayToString(const std::vector<T>& arr, size_t n, cha
 template<typename T, bool is_float>
 struct __StringToTHelper {
   T operator()(const std::string& str) const {
-    return static_cast<T>(std::stoll(str));
+    T ret = 0;
+    Atoi(str.c_str(), &ret);
+    return ret;
   }
 };
 
@@ -350,23 +472,6 @@ struct __StringToTHelper<T, true> {
   }
 };
 
-template<typename T>
-inline static std::vector<T> StringToArray(const std::string& str, char delimiter, size_t n) {
-  if (n == 0) {
-    return std::vector<T>();
-  }
-  std::vector<std::string> strs = Split(str.c_str(), delimiter);
-  if (strs.size() != n) {
-    Log::Fatal("StringToArray error, size doesn't match.");
-  }
-  std::vector<T> ret(n);
-  __StringToTHelper<T, std::is_floating_point<T>::value> helper;
-  for (size_t i = 0; i < n; ++i) {
-    ret[i] = helper(strs[i]);
-  }
-  return ret;
-}
-
 template<typename T>
 inline static std::vector<T> StringToArray(const std::string& str, char delimiter) {
   std::vector<std::string> strs = Split(str.c_str(), delimiter);
@@ -379,6 +484,53 @@ inline static std::vector<T> StringToArray(const std::string& str, char delimite
   return ret;
 }
 
+template<typename T>
+inline static std::vector<T> StringToArray(const std::string& str, int n) {
+  if (n == 0) {
+    return std::vector<T>();
+  }
+  std::vector<std::string> strs = Split(str.c_str(), ' ');
+  CHECK(strs.size() == static_cast<size_t>(n));
+  std::vector<T> ret;
+  ret.reserve(strs.size());
+  __StringToTHelper<T, std::is_floating_point<T>::value> helper;
+  for (const auto& s : strs) {
+    ret.push_back(helper(s));
+  }
+  return ret;
+}
+
+template<typename T, bool is_float>
+struct __StringToTHelperFast {
+  const char* operator()(const char*p, T* out) const {
+    return Atoi(p, out);
+  }
+};
+
+template<typename T>
+struct __StringToTHelperFast<T, true> {
+  const char* operator()(const char*p, T* out) const {
+    double tmp = 0.0f;
+    auto ret = Atof(p, &tmp);
+    *out= static_cast<T>(tmp);
+    return ret;
+  }
+};
+
+template<typename T>
+inline static std::vector<T> StringToArrayFast(const std::string& str, int n) {
+  if (n == 0) {
+    return std::vector<T>();
+  }
+  auto p_str = str.c_str();
+  __StringToTHelperFast<T, std::is_floating_point<T>::value> helper;
+  std::vector<T> ret(n);
+  for (int i = 0; i < n; ++i) {
+    p_str = helper(p_str, &ret[i]);
+  }
+  return ret;
+}
+
 template<typename T>
 inline static std::string Join(const std::vector<T>& strs, const char* delimiter) {
   if (strs.empty()) {
@@ -411,7 +563,7 @@ inline static std::string Join(const std::vector<T>& strs, size_t start, size_t
   return str_buf.str();
 }
 
-static inline int64_t Pow2RoundUp(int64_t x) {
+inline static int64_t Pow2RoundUp(int64_t x) {
   int64_t t = 1;
   for (int i = 0; i < 64; ++i) {
     if (t >= x) {
@@ -426,7 +578,7 @@ static inline int64_t Pow2RoundUp(int64_t x) {
  * \brief Do inplace softmax transformaton on p_rec
  * \param p_rec The input/output vector of the values.
  */
-inline void Softmax(std::vector<double>* p_rec) {
+inline static void Softmax(std::vector<double>* p_rec) {
   std::vector<double> &rec = *p_rec;
   double wmax = rec[0];
   for (size_t i = 1; i < rec.size(); ++i) {
@@ -442,7 +594,7 @@ inline void Softmax(std::vector<double>* p_rec) {
   }
 }
 
-inline void Softmax(const double* input, double* output, int len) {
+inline static void Softmax(const double* input, double* output, int len) {
   double wmax = input[0];
   for (int i = 1; i < len; ++i) {
     wmax = std::max(input[i], wmax);
@@ -467,7 +619,7 @@ std::vector<const T*> ConstPtrInVectorWrapper(const std::vector<std::unique_ptr<
 }
 
 template<typename T1, typename T2>
-inline void SortForPair(std::vector<T1>& keys, std::vector<T2>& values, size_t start, bool is_reverse = false) {
+inline static void SortForPair(std::vector<T1>& keys, std::vector<T2>& values, size_t start, bool is_reverse = false) {
   std::vector<std::pair<T1, T2>> arr;
   for (size_t i = start; i < keys.size(); ++i) {
     arr.emplace_back(keys[i], values[i]);
@@ -537,12 +689,22 @@ inline static double AvoidInf(double x) {
   }
 }
 
-template<class _Iter> inline
+inline static float AvoidInf(float x) {
+	if (x >= 1e38) {
+		return 1e38f;
+	} else if (x <= -1e38) {
+		return -1e38f;
+	} else {
+		return x;
+	}
+}
+
+template<typename _Iter> inline
 static typename std::iterator_traits<_Iter>::value_type* IteratorValType(_Iter) {
   return (0);
 }
 
-template<class _RanIt, class _Pr, class _VTRanIt> inline
+template<typename _RanIt, typename _Pr, typename _VTRanIt> inline
 static void ParallelSort(_RanIt _First, _RanIt _Last, _Pr _Pred, _VTRanIt*) {
   size_t len = _Last - _First;
   const size_t kMinInnerLen = 1024;
@@ -589,14 +751,14 @@ static void ParallelSort(_RanIt _First, _RanIt _Last, _Pr _Pred, _VTRanIt*) {
   }
 }
 
-template<class _RanIt, class _Pr> inline
+template<typename _RanIt, typename _Pr> inline
 static void ParallelSort(_RanIt _First, _RanIt _Last, _Pr _Pred) {
   return ParallelSort(_First, _Last, _Pred, IteratorValType(_First));
 }
 
 // Check that all y[] are in interval [ymin, ymax] (end points included); throws error if not
 template <typename T>
-inline void CheckElementsIntervalClosed(const T *y, T ymin, T ymax, int ny, const char *callername) {
+inline static void CheckElementsIntervalClosed(const T *y, T ymin, T ymax, int ny, const char *callername) {
   auto fatal_msg = [&y, &ymin, &ymax, &callername](int i) { 
     std::ostringstream os;
     os << "[%s]: does not tolerate element [#%i = " << y[i] << "] outside [" << ymin << ", " << ymax << "]";
@@ -627,7 +789,7 @@ inline void CheckElementsIntervalClosed(const T *y, T ymin, T ymax, int ny, cons
 // One-pass scan over array w with nw elements: find min, max and sum of elements;
 // this is useful for checking weight requirements.
 template <typename T1, typename T2>
-inline void ObtainMinMaxSum(const T1 *w, int nw, T1 *mi, T1 *ma, T2 *su) {
+inline static void ObtainMinMaxSum(const T1 *w, int nw, T1 *mi, T1 *ma, T2 *su) {
   T1 minw;
   T1 maxw;
   T1 sumw;
@@ -669,8 +831,8 @@ inline void ObtainMinMaxSum(const T1 *w, int nw, T1 *mi, T1 *ma, T2 *su) {
   }
 }
 
-template<class T>
-inline std::vector<uint32_t> ConstructBitset(const T* vals, int n) {
+template<typename T>
+inline static std::vector<uint32_t> ConstructBitset(const T* vals, int n) {
   std::vector<uint32_t> ret;
   for (int i = 0; i < n; ++i) {
     int i1 = vals[i] / 32;
@@ -683,8 +845,8 @@ inline std::vector<uint32_t> ConstructBitset(const T* vals, int n) {
   return ret;
 }
 
-template<class T>
-inline bool FindInBitset(const uint32_t* bits, int n, T pos) {
+template<typename T>
+inline static bool FindInBitset(const uint32_t* bits, int n, T pos) {
   int i1 = pos / 32;
   if (i1 >= n) {
     return false;
@@ -702,6 +864,24 @@ inline static double GetDoubleUpperBound(double a) {
   return std::nextafter(a, INFINITY);;
 }
 
+inline static size_t GetLine(const char* str) {
+  auto start = str;
+  while (*str != '\0' && *str != '\n' && *str != '\r') {
+    ++str;
+  }
+  return str - start;
+}
+
+inline static const char* SkipNewLine(const char* str) {
+  if (*str == '\r') {
+    ++str;
+  }
+  if (*str == '\n') {
+    ++str;
+  }
+  return str;
+}
+
 }  // namespace Common
 
 }  // namespace LightGBM
