@@ -103,6 +103,8 @@ void GetTaskType(const std::unordered_map<std::string, std::string>& params, Tas
       *task_type = TaskType::kPredict;
     } else if (value == std::string("convert_model")) {
       *task_type = TaskType::kConvertModel;
+    } else if (value == std::string("refit") || value == std::string("refit_tree")) {
+      *task_type = TaskType::KRefitTree;
     } else {
       Log::Fatal("Unknown task type %s", value.c_str());
     }
@@ -269,7 +271,6 @@ void IOConfig::Set(const std::unordered_map<std::string, std::string>& params) {
   GetString(params, "input_model", &input_model);
   GetString(params, "convert_model", &convert_model);
   GetString(params, "output_result", &output_result);
-  GetString(params, "model_format", &model_format);
   std::string tmp_str = "";
   if (GetString(params, "valid_data", &tmp_str)) {
     valid_data_filenames = Common::Split(tmp_str.c_str(), ',');
