@@ -31,13 +31,13 @@ Metadata::~Metadata() {
 
 void Metadata::Init(data_size_t num_data, int weight_idx, int query_idx) {
   num_data_ = num_data;
-  label_ = std::vector<float>(num_data_);
+  label_ = std::vector<label_t>(num_data_);
   if (weight_idx >= 0) {
     if (!weights_.empty()) {
       Log::Info("Using weights in data file, ignoring the additional weights file");
       weights_.clear();
     }
-    weights_ = std::vector<float>(num_data_);
+    weights_ = std::vector<label_t>(num_data_);
     num_weights_ = num_data_;
 #pragma omp parallel for schedule(static)
     for (data_size_t i = 0; i < num_weights_; ++i) {
@@ -63,14 +63,14 @@ void Metadata::Init(data_size_t num_data, int weight_idx, int query_idx) {
 void Metadata::Init(const Metadata& fullset, const data_size_t* used_indices, data_size_t num_used_indices) {
   num_data_ = num_used_indices;
 
-  label_ = std::vector<float>(num_used_indices);
+  label_ = std::vector<label_t>(num_used_indices);
 #pragma omp parallel for schedule(static)
   for (data_size_t i = 0; i < num_used_indices; i++) {
     label_[i] = fullset.label_[used_indices[i]];
   }
 
   if (!fullset.weights_.empty()) {
-    weights_ = std::vector<float>(num_used_indices);
+    weights_ = std::vector<label_t>(num_used_indices);
     num_weights_ = num_used_indices;
 #pragma omp parallel for schedule(static)
     for (data_size_t i = 0; i < num_used_indices; i++) {
@@ -134,7 +134,7 @@ void Metadata::PartitionLabel(const std::vector<data_size_t>& used_indices) {
   }
   auto old_label = label_;
   num_data_ = static_cast<data_size_t>(used_indices.size());
-  label_ = std::vector<float>(num_data_);
+  label_ = std::vector<label_t>(num_data_);
 #pragma omp parallel for schedule(static)
   for (data_size_t i = 0; i < num_data_; ++i) {
     label_[i] = old_label[used_indices[i]];
@@ -205,7 +205,7 @@ void Metadata::CheckOrPartition(data_size_t num_all_data, const std::vector<data
       if (!weights_.empty()) {
         auto old_weights = weights_;
         num_weights_ = num_data_;
-        weights_ = std::vector<float>(num_data_);
+        weights_ = std::vector<label_t>(num_data_);
 #pragma omp parallel for schedule(static)
         for (int i = 0; i < static_cast<int>(used_data_indices.size()); ++i) {
           weights_[i] = old_weights[used_data_indices[i]];
@@ -302,7 +302,7 @@ void Metadata::SetInitScore(const double* init_score, data_size_t len) {
   init_score_load_from_file_ = false;
 }
 
-void Metadata::SetLabel(const float* label, data_size_t len) {
+void Metadata::SetLabel(const label_t* label, data_size_t len) {
   std::lock_guard<std::mutex> lock(mutex_);
   if (label == nullptr) {
     Log::Fatal("label cannot be nullptr");
@@ -311,14 +311,14 @@ void Metadata::SetLabel(const float* label, data_size_t len) {
     Log::Fatal("len of label is not same with #data");
   }
   if (!label_.empty()) { label_.clear(); }
-  label_ = std::vector<float>(num_data_);
+  label_ = std::vector<label_t>(num_data_);
 #pragma omp parallel for schedule(static)
   for (data_size_t i = 0; i < num_data_; ++i) {
     label_[i] = label[i];
   }
 }
 
-void Metadata::SetWeights(const float* weights, data_size_t len) {
+void Metadata::SetWeights(const label_t* weights, data_size_t len) {
   std::lock_guard<std::mutex> lock(mutex_);
   // save to nullptr
   if (weights == nullptr || len == 0) {
@@ -331,7 +331,7 @@ void Metadata::SetWeights(const float* weights, data_size_t len) {
   }
   if (!weights_.empty()) { weights_.clear(); }
   num_weights_ = num_data_;
-  weights_ = std::vector<float>(num_weights_);
+  weights_ = std::vector<label_t>(num_weights_);
 #pragma omp parallel for schedule(static)
   for (data_size_t i = 0; i < num_weights_; ++i) {
     weights_[i] = weights[i];
@@ -379,12 +379,12 @@ void Metadata::LoadWeights() {
   }
   Log::Info("Loading weights...");
   num_weights_ = static_cast<data_size_t>(reader.Lines().size());
-  weights_ = std::vector<float>(num_weights_);
+  weights_ = std::vector<label_t>(num_weights_);
 #pragma omp parallel for schedule(static)
   for (data_size_t i = 0; i < num_weights_; ++i) {
     double tmp_weight = 0.0f;
     Common::Atof(reader.Lines()[i].c_str(), &tmp_weight);
-    weights_[i] = static_cast<float>(tmp_weight);
+    weights_[i] = static_cast<label_t>(tmp_weight);
   }
   weight_load_from_file_ = true;
 }
@@ -463,7 +463,7 @@ void Metadata::LoadQueryWeights() {
   }
   query_weights_.clear();
   Log::Info("Loading query weights...");
-  query_weights_ = std::vector<float>(num_queries_);
+  query_weights_ = std::vector<label_t>(num_queries_);
   for (data_size_t i = 0; i < num_queries_; ++i) {
     query_weights_[i] = 0.0f;
     for (data_size_t j = query_boundaries_[i]; j < query_boundaries_[i + 1]; ++j) {
@@ -484,15 +484,15 @@ void Metadata::LoadFromMemory(const void* memory) {
   mem_ptr += sizeof(num_queries_);
 
   if (!label_.empty()) { label_.clear(); }
-  label_ = std::vector<float>(num_data_);
-  std::memcpy(label_.data(), mem_ptr, sizeof(float)*num_data_);
-  mem_ptr += sizeof(float)*num_data_;
+  label_ = std::vector<label_t>(num_data_);
+  std::memcpy(label_.data(), mem_ptr, sizeof(label_t)*num_data_);
+  mem_ptr += sizeof(label_t)*num_data_;
 
   if (num_weights_ > 0) {
     if (!weights_.empty()) { weights_.clear(); }
-    weights_ = std::vector<float>(num_weights_);
-    std::memcpy(weights_.data(), mem_ptr, sizeof(float)*num_weights_);
-    mem_ptr += sizeof(float)*num_weights_;
+    weights_ = std::vector<label_t>(num_weights_);
+    std::memcpy(weights_.data(), mem_ptr, sizeof(label_t)*num_weights_);
+    mem_ptr += sizeof(label_t)*num_weights_;
     weight_load_from_file_ = true;
   }
   if (num_queries_ > 0) {
@@ -509,9 +509,9 @@ void Metadata::SaveBinaryToFile(FILE* file) const {
   fwrite(&num_data_, sizeof(num_data_), 1, file);
   fwrite(&num_weights_, sizeof(num_weights_), 1, file);
   fwrite(&num_queries_, sizeof(num_queries_), 1, file);
-  fwrite(label_.data(), sizeof(float), num_data_, file);
+  fwrite(label_.data(), sizeof(label_t), num_data_, file);
   if (!weights_.empty()) {
-    fwrite(weights_.data(), sizeof(float), num_weights_, file);
+    fwrite(weights_.data(), sizeof(label_t), num_weights_, file);
   }
   if (!query_boundaries_.empty()) {
     fwrite(query_boundaries_.data(), sizeof(data_size_t), num_queries_ + 1, file);
@@ -522,9 +522,9 @@ void Metadata::SaveBinaryToFile(FILE* file) const {
 size_t Metadata::SizesInByte() const  {
   size_t size = sizeof(num_data_) + sizeof(num_weights_)
     + sizeof(num_queries_);
-  size += sizeof(float) * num_data_;
+  size += sizeof(label_t) * num_data_;
   if (!weights_.empty()) {
-    size += sizeof(float) * num_weights_;
+    size += sizeof(label_t) * num_weights_;
   }
   if (!query_boundaries_.empty()) {
     size += sizeof(data_size_t) * (num_queries_ + 1);
