@@ -49,7 +49,7 @@ Tree::~Tree() {
 
 int Tree::Split(int leaf, int feature, int real_feature, uint32_t threshold_bin,
                 double threshold_double, double left_value, double right_value,
-                data_size_t left_cnt, data_size_t right_cnt, double gain, MissingType missing_type, bool default_left) {
+                int left_cnt, int right_cnt, float gain, MissingType missing_type, bool default_left) {
   Split(leaf, feature, real_feature, left_value, right_value, left_cnt, right_cnt, gain);
   int new_node_idx = num_leaves_ - 1;
   decision_type_[new_node_idx] = 0;
@@ -70,7 +70,7 @@ int Tree::Split(int leaf, int feature, int real_feature, uint32_t threshold_bin,
 
 int Tree::SplitCategorical(int leaf, int feature, int real_feature, const uint32_t* threshold_bin, int num_threshold_bin,
                            const uint32_t* threshold, int num_threshold, double left_value, double right_value,
-                           data_size_t left_cnt, data_size_t right_cnt, double gain, MissingType missing_type) {
+                           data_size_t left_cnt, data_size_t right_cnt, float gain, MissingType missing_type) {
   Split(leaf, feature, real_feature, left_value, right_value, left_cnt, right_cnt, gain);
   int new_node_idx = num_leaves_ - 1;
   decision_type_[new_node_idx] = 0;
@@ -207,49 +207,49 @@ void Tree::AddPredictionToScore(const Dataset* data,
 
 std::string Tree::ToString() const {
   std::stringstream str_buf;
-  str_buf << "num_leaves=" << num_leaves_ << std::endl;
-  str_buf << "num_cat=" << num_cat_ << std::endl;
+  str_buf << "num_leaves=" << num_leaves_ << '\n';
+  str_buf << "num_cat=" << num_cat_ << '\n';
   str_buf << "split_feature="
-    << Common::ArrayToString<int>(split_feature_, num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToStringFast(split_feature_, num_leaves_ - 1) << '\n';
   str_buf << "split_gain="
-    << Common::ArrayToString<double>(split_gain_, num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToStringFast(split_gain_, num_leaves_ - 1) << '\n';
   str_buf << "threshold="
-    << Common::ArrayToString<double>(threshold_, num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToString(threshold_, num_leaves_ - 1) << '\n';
   str_buf << "decision_type="
-    << Common::ArrayToString<int>(Common::ArrayCast<int8_t, int>(decision_type_), num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToStringFast(Common::ArrayCast<int8_t, int>(decision_type_), num_leaves_ - 1) << '\n';
   str_buf << "left_child="
-    << Common::ArrayToString<int>(left_child_, num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToStringFast(left_child_, num_leaves_ - 1) << '\n';
   str_buf << "right_child="
-    << Common::ArrayToString<int>(right_child_, num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToStringFast(right_child_, num_leaves_ - 1) << '\n';
   str_buf << "leaf_value="
-    << Common::ArrayToString<double>(leaf_value_, num_leaves_, ' ') << std::endl;
+    << Common::ArrayToString(leaf_value_, num_leaves_) << '\n';
   str_buf << "leaf_count="
-    << Common::ArrayToString<data_size_t>(leaf_count_, num_leaves_, ' ') << std::endl;
+    << Common::ArrayToStringFast(leaf_count_, num_leaves_) << '\n';
   str_buf << "internal_value="
-    << Common::ArrayToString<double>(internal_value_, num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToStringFast(internal_value_, num_leaves_ - 1) << '\n';
   str_buf << "internal_count="
-    << Common::ArrayToString<data_size_t>(internal_count_, num_leaves_ - 1, ' ') << std::endl;
+    << Common::ArrayToStringFast(internal_count_, num_leaves_ - 1) << '\n';
   if (num_cat_ > 0) {
     str_buf << "cat_boundaries="
-      << Common::ArrayToString<int>(cat_boundaries_, num_cat_ + 1, ' ') << std::endl;
+      << Common::ArrayToStringFast(cat_boundaries_, num_cat_ + 1) << '\n';
     str_buf << "cat_threshold="
-      << Common::ArrayToString<uint32_t>(cat_threshold_, cat_threshold_.size(), ' ') << std::endl;
+      << Common::ArrayToStringFast(cat_threshold_, cat_threshold_.size()) << '\n';
   }
-  str_buf << "shrinkage=" << shrinkage_ << std::endl;
-  str_buf << std::endl;
+  str_buf << "shrinkage=" << shrinkage_ << '\n';
+  str_buf << '\n';
   return str_buf.str();
 }
 
 std::string Tree::ToJSON() const {
   std::stringstream str_buf;
   str_buf << std::setprecision(std::numeric_limits<double>::digits10 + 2);
-  str_buf << "\"num_leaves\":" << num_leaves_ << "," << std::endl;
-  str_buf << "\"num_cat\":" << num_cat_ << "," << std::endl;
-  str_buf << "\"shrinkage\":" << shrinkage_ << "," << std::endl;
+  str_buf << "\"num_leaves\":" << num_leaves_ << "," << '\n';
+  str_buf << "\"num_cat\":" << num_cat_ << "," << '\n';
+  str_buf << "\"shrinkage\":" << shrinkage_ << "," << '\n';
   if (num_leaves_ == 1) {
-    str_buf << "\"tree_structure\":{" << "\"leaf_value\":" << leaf_value_[0] << "}" << std::endl;
+    str_buf << "\"tree_structure\":{" << "\"leaf_value\":" << leaf_value_[0] << "}" << '\n';
   } else {
-    str_buf << "\"tree_structure\":" << NodeToJSON(0) << std::endl;
+    str_buf << "\"tree_structure\":" << NodeToJSON(0) << '\n';
   }
 
   return str_buf.str();
@@ -260,10 +260,10 @@ std::string Tree::NodeToJSON(int index) const {
   str_buf << std::setprecision(std::numeric_limits<double>::digits10 + 2);
   if (index >= 0) {
     // non-leaf
-    str_buf << "{" << std::endl;
-    str_buf << "\"split_index\":" << index << "," << std::endl;
-    str_buf << "\"split_feature\":" << split_feature_[index] << "," << std::endl;
-    str_buf << "\"split_gain\":" << split_gain_[index] << "," << std::endl;
+    str_buf << "{" << '\n';
+    str_buf << "\"split_index\":" << index << "," << '\n';
+    str_buf << "\"split_feature\":" << split_feature_[index] << "," << '\n';
+    str_buf << "\"split_gain\":" << split_gain_[index] << "," << '\n';
     if (GetDecisionType(decision_type_[index], kCategoricalMask)) {
       int cat_idx = static_cast<int>(threshold_[index]);
       std::vector<int> cats;
@@ -276,37 +276,37 @@ std::string Tree::NodeToJSON(int index) const {
           }
         }
       }
-      str_buf << "\"threshold\":\"" << Common::Join(cats, "||") << "\"," << std::endl;
-      str_buf << "\"decision_type\":\"==\"," << std::endl;
+      str_buf << "\"threshold\":\"" << Common::Join(cats, "||") << "\"," << '\n';
+      str_buf << "\"decision_type\":\"==\"," << '\n';
     } else {
-      str_buf << "\"threshold\":" << Common::AvoidInf(threshold_[index]) << "," << std::endl;
-      str_buf << "\"decision_type\":\"<=\"," << std::endl;
+      str_buf << "\"threshold\":" << Common::AvoidInf(threshold_[index]) << "," << '\n';
+      str_buf << "\"decision_type\":\"<=\"," << '\n';
     }
     if (GetDecisionType(decision_type_[index], kDefaultLeftMask)) {
-      str_buf << "\"default_left\":true," << std::endl;
+      str_buf << "\"default_left\":true," << '\n';
     } else {
-      str_buf << "\"default_left\":false," << std::endl;
+      str_buf << "\"default_left\":false," << '\n';
     }
     uint8_t missing_type = GetMissingType(decision_type_[index]);
     if (missing_type == 0) {
-      str_buf << "\"missing_type\":\"None\"," << std::endl;
+      str_buf << "\"missing_type\":\"None\"," << '\n';
     } else if (missing_type == 1) {
-      str_buf << "\"missing_type\":\"Zero\"," << std::endl;
+      str_buf << "\"missing_type\":\"Zero\"," << '\n';
     } else {
-      str_buf << "\"missing_type\":\"NaN\"," << std::endl;
+      str_buf << "\"missing_type\":\"NaN\"," << '\n';
     }
-    str_buf << "\"internal_value\":" << internal_value_[index] << "," << std::endl;
-    str_buf << "\"internal_count\":" << internal_count_[index] << "," << std::endl;
-    str_buf << "\"left_child\":" << NodeToJSON(left_child_[index]) << "," << std::endl;
-    str_buf << "\"right_child\":" << NodeToJSON(right_child_[index]) << std::endl;
+    str_buf << "\"internal_value\":" << internal_value_[index] << "," << '\n';
+    str_buf << "\"internal_count\":" << internal_count_[index] << "," << '\n';
+    str_buf << "\"left_child\":" << NodeToJSON(left_child_[index]) << "," << '\n';
+    str_buf << "\"right_child\":" << NodeToJSON(right_child_[index]) << '\n';
     str_buf << "}";
   } else {
     // leaf
     index = ~index;
-    str_buf << "{" << std::endl;
-    str_buf << "\"leaf_index\":" << index << "," << std::endl;
-    str_buf << "\"leaf_value\":" << leaf_value_[index] << "," << std::endl;
-    str_buf << "\"leaf_count\":" << leaf_count_[index] << std::endl;
+    str_buf << "{" << '\n';
+    str_buf << "\"leaf_index\":" << index << "," << '\n';
+    str_buf << "\"leaf_value\":" << leaf_value_[index] << "," << '\n';
+    str_buf << "\"leaf_count\":" << leaf_count_[index] << '\n';
     str_buf << "}";
   }
 
@@ -317,7 +317,7 @@ std::string Tree::NumericalDecisionIfElse(int node) const {
   std::stringstream str_buf;
   uint8_t missing_type = GetMissingType(decision_type_[node]);
   bool default_left = GetDecisionType(decision_type_[node], kDefaultLeftMask);
-  if (missing_type == 0 || (missing_type == 1 && default_left && kZeroAsMissingValueRange < threshold_[node])) {
+  if (missing_type == 0 || (missing_type == 1 && default_left && kZeroThreshold < threshold_[node])) {
     str_buf << "if (fval <= " << threshold_[node] << ") {";
   } else if (missing_type == 1) {
     if (default_left) {
@@ -376,7 +376,7 @@ std::string Tree::ToIfElse(int index, bool is_predict_leaf_index) const {
     }
     str_buf << NodeToIfElse(0, is_predict_leaf_index);
   }
-  str_buf << " }" << std::endl;
+  str_buf << " }" << '\n';
 
   //Predict func by Map to ifelse
   str_buf << "double PredictTree" << index;
@@ -404,7 +404,7 @@ std::string Tree::ToIfElse(int index, bool is_predict_leaf_index) const {
     }
     str_buf << NodeToIfElseByMap(0, is_predict_leaf_index);
   }
-  str_buf << " }" << std::endl;
+  str_buf << " }" << '\n';
 
   return str_buf.str();
 }
@@ -471,19 +471,26 @@ std::string Tree::NodeToIfElseByMap(int index, bool is_predict_leaf_index) const
   return str_buf.str();
 }
 
-Tree::Tree(const std::string& str) {
-  std::vector<std::string> lines = Common::SplitLines(str.c_str());
+Tree::Tree(const char* str, size_t* used_len) {
+  auto p = str;
   std::unordered_map<std::string, std::string> key_vals;
-  for (const std::string& line : lines) {
-    std::vector<std::string> tmp_strs = Common::Split(line.c_str(), '=');
-    if (tmp_strs.size() == 2) {
-      std::string key = Common::Trim(tmp_strs[0]);
-      std::string val = Common::Trim(tmp_strs[1]);
-      if (key.size() > 0 && val.size() > 0) {
-        key_vals[key] = val;
-      }
-    }
+  const int max_num_line = 15;
+  int read_line = 0;
+  while (read_line < max_num_line) {
+    if (*p == '\r' || *p == '\n') break;
+    auto start = p;
+    while (*p != '=') ++p; 
+    std::string key(start, p - start);
+    ++p;
+    start = p;
+    while (*p != '\r' && *p != '\n') ++p;
+    key_vals[key] = std::string(start, p - start);
+    ++read_line;
+    if (*p == '\r') ++p;
+    if (*p == '\n') ++p;
   }
+  *used_len = p - str;
+
   if (key_vals.count("num_leaves") <= 0) {
     Log::Fatal("Tree model should contain num_leaves field.");
   }
@@ -497,7 +504,7 @@ Tree::Tree(const std::string& str) {
   Common::Atoi(key_vals["num_cat"].c_str(), &num_cat_);
 
   if (key_vals.count("leaf_value")) {
-    leaf_value_ = Common::StringToArray<double>(key_vals["leaf_value"], ' ', num_leaves_);
+    leaf_value_ = Common::StringToArray<double>(key_vals["leaf_value"], num_leaves_);
   } else {
     Log::Fatal("Tree model string format error, should contain leaf_value field");
   }
@@ -505,68 +512,68 @@ Tree::Tree(const std::string& str) {
   if (num_leaves_ <= 1) { return; }
 
   if (key_vals.count("left_child")) {
-    left_child_ = Common::StringToArray<int>(key_vals["left_child"], ' ', num_leaves_ - 1);
+    left_child_ = Common::StringToArrayFast<int>(key_vals["left_child"], num_leaves_ - 1);
   } else {
     Log::Fatal("Tree model string format error, should contain left_child field");
   }
 
   if (key_vals.count("right_child")) {
-    right_child_ = Common::StringToArray<int>(key_vals["right_child"], ' ', num_leaves_ - 1);
+    right_child_ = Common::StringToArrayFast<int>(key_vals["right_child"], num_leaves_ - 1);
   } else {
     Log::Fatal("Tree model string format error, should contain right_child field");
   }
 
   if (key_vals.count("split_feature")) {
-    split_feature_ = Common::StringToArray<int>(key_vals["split_feature"], ' ', num_leaves_ - 1);
+    split_feature_ = Common::StringToArrayFast<int>(key_vals["split_feature"], num_leaves_ - 1);
   } else {
     Log::Fatal("Tree model string format error, should contain split_feature field");
   }
 
   if (key_vals.count("threshold")) {
-    threshold_ = Common::StringToArray<double>(key_vals["threshold"], ' ', num_leaves_ - 1);
+    threshold_ = Common::StringToArray<double>(key_vals["threshold"], num_leaves_ - 1);
   } else {
     Log::Fatal("Tree model string format error, should contain threshold field");
   }
 
   if (key_vals.count("split_gain")) {
-    split_gain_ = Common::StringToArray<double>(key_vals["split_gain"], ' ', num_leaves_ - 1);
+    split_gain_ = Common::StringToArrayFast<float>(key_vals["split_gain"], num_leaves_ - 1);
   } else {
     split_gain_.resize(num_leaves_ - 1);
   }
 
   if (key_vals.count("internal_count")) {
-    internal_count_ = Common::StringToArray<data_size_t>(key_vals["internal_count"], ' ', num_leaves_ - 1);
+    internal_count_ = Common::StringToArrayFast<int>(key_vals["internal_count"], num_leaves_ - 1);
   } else {
     internal_count_.resize(num_leaves_ - 1);
   }
 
   if (key_vals.count("internal_value")) {
-    internal_value_ = Common::StringToArray<double>(key_vals["internal_value"], ' ', num_leaves_ - 1);
+    internal_value_ = Common::StringToArrayFast<double>(key_vals["internal_value"], num_leaves_ - 1);
   } else {
     internal_value_.resize(num_leaves_ - 1);
   }
 
   if (key_vals.count("leaf_count")) {
-    leaf_count_ = Common::StringToArray<data_size_t>(key_vals["leaf_count"], ' ', num_leaves_);
+    leaf_count_ = Common::StringToArrayFast<int>(key_vals["leaf_count"], num_leaves_);
   } else {
     leaf_count_.resize(num_leaves_);
   }
 
   if (key_vals.count("decision_type")) {
-    decision_type_ = Common::StringToArray<int8_t>(key_vals["decision_type"], ' ', num_leaves_ - 1);
+    decision_type_ = Common::StringToArrayFast<int8_t>(key_vals["decision_type"], num_leaves_ - 1);
   } else {
     decision_type_ = std::vector<int8_t>(num_leaves_ - 1, 0);
   }
 
   if (num_cat_ > 0) {
     if (key_vals.count("cat_boundaries")) {
-      cat_boundaries_ = Common::StringToArray<int>(key_vals["cat_boundaries"], ' ', num_cat_ + 1);
+      cat_boundaries_ = Common::StringToArrayFast<int>(key_vals["cat_boundaries"], num_cat_ + 1);
     } else {
       Log::Fatal("Tree model should contain cat_boundaries field.");
     }
 
     if (key_vals.count("cat_threshold")) {
-      cat_threshold_ = Common::StringToArray<uint32_t>(key_vals["cat_threshold"], ' ', cat_boundaries_.back());
+      cat_threshold_ = Common::StringToArrayFast<uint32_t>(key_vals["cat_threshold"], cat_boundaries_.back());
     } else {
       Log::Fatal("Tree model should contain cat_threshold field.");
     }
