@@ -52,13 +52,13 @@ public:
     weights_ = metadata.weights();
 
     CHECK_NOTNULL(label_);
-    Common::CheckElementsIntervalClosed(label_, 0.0f, 1.0f, num_data_, GetName());
+    Common::CheckElementsIntervalClosed<label_t>(label_, 0.0f, 1.0f, num_data_, GetName());
     Log::Info("[%s:%s]: (objective) labels passed interval [0, 1] check",  GetName(), __func__);
 
     if (weights_ != nullptr) {
-      float minw;
+      label_t minw;
       double sumw;
-      Common::ObtainMinMaxSum(weights_, num_data_, &minw, (float*)nullptr, &sumw);
+      Common::ObtainMinMaxSum(weights_, num_data_, &minw, (label_t*)nullptr, &sumw);
       if (minw < 0.0f) {
         Log::Fatal("[%s]: at least one weight is negative.", GetName());
       }
@@ -133,9 +133,9 @@ private:
   /*! \brief Number of data points */
   data_size_t num_data_;
   /*! \brief Pointer for label */
-  const float* label_;
+  const label_t* label_;
   /*! \brief Weights for data */
-  const float* weights_;
+  const label_t* weights_;
 };
 
 /*!
@@ -158,12 +158,12 @@ public:
     weights_ = metadata.weights();
 
     CHECK_NOTNULL(label_);
-    Common::CheckElementsIntervalClosed(label_, 0.0f, 1.0f, num_data_, GetName());
+    Common::CheckElementsIntervalClosed<label_t>(label_, 0.0f, 1.0f, num_data_, GetName());
     Log::Info("[%s:%s]: (objective) labels passed interval [0, 1] check",  GetName(), __func__);
 
     if (weights_ != nullptr) {
 
-      Common::ObtainMinMaxSum(weights_, num_data_, &min_weight_, &max_weight_, (float*)nullptr);
+      Common::ObtainMinMaxSum(weights_, num_data_, &min_weight_, &max_weight_, (label_t*)nullptr);
       if (min_weight_ <= 0.0f) {
         Log::Fatal("[%s]: at least one weight is non-positive.", GetName());
       }
@@ -254,13 +254,13 @@ private:
   /*! \brief Number of data points */
   data_size_t num_data_;
   /*! \brief Pointer for label */
-  const float* label_;
+  const label_t* label_;
   /*! \brief Weights for data */
-  const float* weights_;
+  const label_t* weights_;
   /*! \brief Minimum weight found during init */
-  float min_weight_;
+  label_t min_weight_;
   /*! \brief Maximum weight found during init */
-  float max_weight_;
+  label_t max_weight_;
 };
 
 }  // end namespace LightGBM
