@@ -346,8 +346,11 @@ class _InnerPredictor(object):
         self.pred_parameter = param_dict_to_str(pred_parameter)
 
     def __del__(self):
-        if self.__is_manage_handle:
-            _safe_call(_LIB.LGBM_BoosterFree(self.handle))
+        try:
+            if self.__is_manage_handle:
+                _safe_call(_LIB.LGBM_BoosterFree(self.handle))
+        except AttributeError:
+            pass
 
     def __getstate__(self):
         this = self.__dict__.copy()
@@ -555,7 +558,7 @@ class _InnerPredictor(object):
 
 class Dataset(object):
     """Dataset in LightGBM."""
-    def __init__(self, data, label=None, max_bin=None, reference=None,
+    def __init__(self, data, label=None, reference=None,
                  weight=None, group=None, init_score=None, silent=False,
                  feature_name='auto', categorical_feature='auto', params=None,
                  free_raw_data=True):
@@ -568,9 +571,6 @@ class Dataset(object):
             If string, it represents the path to txt file.
         label : list, numpy 1-D array or None, optional (default=None)
             Label of the data.
-        max_bin : int or None, optional (default=None)
-            Max number of discrete bins for features.
-            If None, default value from parameters of CLI-version will be used.
         reference : Dataset or None, optional (default=None)
             If this is Dataset for validation, training data should be used as reference.
         weight : list, numpy 1-D array or None, optional (default=None)
@@ -597,7 +597,6 @@ class Dataset(object):
         self.handle = None
         self.data = data
         self.label = label
-        self.max_bin = max_bin
         self.reference = reference
         self.weight = weight
         self.group = group
@@ -613,14 +612,17 @@ class Dataset(object):
         self.params_back_up = None
 
     def __del__(self):
-        self._free_handle()
+        try:
+            self._free_handle()
+        except AttributeError:
+            pass
 
     def _free_handle(self):
         if self.handle is not None:
             _safe_call(_LIB.LGBM_DatasetFree(self.handle))
             self.handle = None
 
-    def _lazy_init(self, data, label=None, max_bin=None, reference=None,
+    def _lazy_init(self, data, label=None, reference=None,
                    weight=None, group=None, init_score=None, predictor=None,
                    silent=False, feature_name='auto',
                    categorical_feature='auto', params=None):
@@ -640,12 +642,7 @@ class Dataset(object):
             if key in args_names:
                 warnings.warn('{0} keyword has been found in `params` and will be ignored. '
                               'Please use {0} argument of the Dataset constructor to pass this parameter.'.format(key))
-        self.max_bin = max_bin
         self.predictor = predictor
-        if self.max_bin is not None:
-            params["max_bin"] = self.max_bin
-            warnings.warn('The `max_bin` parameter is deprecated and will be removed in 2.0.12 version. '
-                          'Please use `params` to pass this parameter.', LGBMDeprecationWarning)
         if "verbosity" in params:
             params.setdefault("verbose", params.pop("verbosity"))
         if silent:
@@ -821,7 +818,7 @@ class Dataset(object):
             if self.reference is not None:
                 if self.used_indices is None:
                     # create valid
-                    self._lazy_init(self.data, label=self.label, max_bin=self.max_bin, reference=self.reference,
+                    self._lazy_init(self.data, label=self.label, reference=self.reference,
                                     weight=self.weight, group=self.group, init_score=self.init_score, predictor=self._predictor,
                                     silent=self.silent, feature_name=self.feature_name, params=self.params)
                 else:
@@ -839,7 +836,7 @@ class Dataset(object):
                         raise ValueError("Label should not be None.")
             else:
                 # create train
-                self._lazy_init(self.data, label=self.label, max_bin=self.max_bin,
+                self._lazy_init(self.data, label=self.label,
                                 weight=self.weight, group=self.group, init_score=self.init_score,
                                 predictor=self._predictor, silent=self.silent, feature_name=self.feature_name,
                                 categorical_feature=self.categorical_feature, params=self.params)
@@ -874,7 +871,7 @@ class Dataset(object):
         self : Dataset
             Returns self.
         """
-        ret = Dataset(data, label=label, max_bin=self.max_bin, reference=self,
+        ret = Dataset(data, label=label, reference=self,
                       weight=weight, group=group, init_score=init_score,
                       silent=silent, params=params, free_raw_data=self.free_raw_data)
         ret._predictor = self._predictor
@@ -989,7 +986,7 @@ class Dataset(object):
             A numpy array with information from the Dataset.
         """
         if self.handle is None:
-            raise Exception("Cannot get %s before construct dataset" % field_name)
+            raise Exception("Cannot get %s before construct Dataset" % field_name)
         tmp_out_len = ctypes.c_int()
         out_type = ctypes.c_int()
         ret = ctypes.POINTER(ctypes.c_void_p)()
@@ -1147,7 +1144,7 @@ class Dataset(object):
         label : numpy array
             The label information from the Dataset.
         """
-        if self.label is None and self.handle is not None:
+        if self.label is None:
             self.label = self.get_field('label')
         return self.label
 
@@ -1159,7 +1156,7 @@ class Dataset(object):
         weight : numpy array
             Weight for each data point from the Dataset.
         """
-        if self.weight is None and self.handle is not None:
+        if self.weight is None:
             self.weight = self.get_field('weight')
         return self.weight
 
@@ -1171,7 +1168,7 @@ class Dataset(object):
         init_score : numpy array
             Init score of Booster.
         """
-        if self.init_score is None and self.handle is not None:
+        if self.init_score is None:
             self.init_score = self.get_field('init_score')
         return self.init_score
 
@@ -1183,7 +1180,7 @@ class Dataset(object):
         group : numpy array
             Group size of each group.
         """
-        if self.group is None and self.handle is not None:
+        if self.group is None:
             self.group = self.get_field('group')
             if self.group is not None:
                 # group data from LightGBM is boundaries data, need to convert to group size
@@ -1348,10 +1345,16 @@ class Booster(object):
             raise TypeError('Need at least one training dataset or model file to create booster instance')
 
     def __del__(self):
-        if self.network:
-            self.free_network()
-        if self.handle is not None:
-            _safe_call(_LIB.LGBM_BoosterFree(self.handle))
+        try:
+            if self.network:
+                self.free_network()
+        except AttributeError:
+            pass
+        try:
+            if self.handle is not None:
+                _safe_call(_LIB.LGBM_BoosterFree(self.handle))
+        except AttributeError:
+            pass
 
     def __copy__(self):
         return self.__deepcopy__(None)
