@@ -15,7 +15,8 @@ from .basic import Dataset, LightGBMError
 from .compat import (SKLEARN_INSTALLED, _LGBMClassifierBase,
                      LGBMNotFittedError, _LGBMLabelEncoder, _LGBMModelBase,
                      _LGBMRegressorBase, _LGBMCheckXY, _LGBMCheckArray, _LGBMCheckConsistentLength,
-                     _LGBMCheckClassificationTargets, argc_, range_, LGBMDeprecationWarning)
+                     _LGBMCheckClassificationTargets, _LGBMComputeSampleWeight,
+                     argc_, range_, LGBMDeprecationWarning)
 from .engine import train
 
 
@@ -133,8 +134,8 @@ class LGBMModel(_LGBMModelBase):
     """Implementation of the scikit-learn API for LightGBM."""
 
     def __init__(self, boosting_type="gbdt", num_leaves=31, max_depth=-1,
-                 learning_rate=0.1, n_estimators=10, max_bin=255,
-                 subsample_for_bin=200000, objective=None,
+                 learning_rate=0.1, n_estimators=100,
+                 subsample_for_bin=200000, objective=None, class_weight=None,
                  min_split_gain=0., min_child_weight=1e-3, min_child_samples=20,
                  subsample=1., subsample_freq=1, colsample_bytree=1.,
                  reg_alpha=0., reg_lambda=0., random_state=None,
@@ -154,16 +155,23 @@ class LGBMModel(_LGBMModelBase):
             Maximum tree depth for base learners, -1 means no limit.
         learning_rate : float, optional (default=0.1)
             Boosting learning rate.
-        n_estimators : int, optional (default=10)
+        n_estimators : int, optional (default=100)
             Number of boosted trees to fit.
-        max_bin : int, optional (default=255)
-            Number of bucketed bins for feature values.
         subsample_for_bin : int, optional (default=50000)
             Number of samples for constructing bins.
         objective : string, callable or None, optional (default=None)
             Specify the learning task and the corresponding learning objective or
             a custom objective function to be used (see note below).
             default: 'regression' for LGBMRegressor, 'binary' or 'multiclass' for LGBMClassifier, 'lambdarank' for LGBMRanker.
+        class_weight : dict, 'balanced' or None, optional (default=None)
+            Weights associated with classes in the form ``{class_label: weight}``.
+            Use this parameter only for multi-class classification task;
+            for binary classification task you may use ``is_unbalance`` or ``scale_pos_weight`` parameters.
+            The 'balanced' mode uses the values of y to automatically adjust weights
+            inversely proportional to class frequencies in the input data as ``n_samples / (n_classes * np.bincount(y))``.
+            If None, all classes are supposed to have weight one.
+            Note that these weights will be multiplied with ``sample_weight`` (passed through the fit method)
+            if ``sample_weight`` is specified.
         min_split_gain : float, optional (default=0.)
             Minimum loss reduction required to make a further partition on a leaf node of the tree.
         min_child_weight : float, optional (default=1e-3)
@@ -246,7 +254,6 @@ class LGBMModel(_LGBMModelBase):
         self.max_depth = max_depth
         self.learning_rate = learning_rate
         self.n_estimators = n_estimators
-        self.max_bin = max_bin
         self.subsample_for_bin = subsample_for_bin
         self.min_split_gain = min_split_gain
         self.min_child_weight = min_child_weight
@@ -265,6 +272,7 @@ class LGBMModel(_LGBMModelBase):
         self._best_iteration = None
         self._other_params = {}
         self._objective = objective
+        self.class_weight = class_weight
         self._n_features = None
         self._classes = None
         self._n_classes = None
@@ -273,12 +281,6 @@ class LGBMModel(_LGBMModelBase):
     def get_params(self, deep=True):
         params = super(LGBMModel, self).get_params(deep=deep)
         params.update(self._other_params)
-        if 'seed' in params:
-            warnings.warn('The `seed` parameter is deprecated and will be removed in 2.0.12 version. '
-                          'Please use `random_state` instead.', LGBMDeprecationWarning)
-        if 'nthread' in params:
-            warnings.warn('The `nthread` parameter is deprecated and will be removed in 2.0.12 version. '
-                          'Please use `n_jobs` instead.', LGBMDeprecationWarning)
         return params
 
     # minor change to support `**kwargs`
@@ -293,9 +295,9 @@ class LGBMModel(_LGBMModelBase):
     def fit(self, X, y,
             sample_weight=None, init_score=None, group=None,
             eval_set=None, eval_names=None, eval_sample_weight=None,
-            eval_init_score=None, eval_group=None, eval_metric=None,
-            early_stopping_rounds=None, verbose=True, feature_name='auto',
-            categorical_feature='auto', callbacks=None):
+            eval_class_weight=None, eval_init_score=None, eval_group=None,
+            eval_metric=None, early_stopping_rounds=None, verbose=True,
+            feature_name='auto', categorical_feature='auto', callbacks=None):
         """Build a gradient boosting model from the training set (X, y).
 
         Parameters
@@ -312,10 +314,12 @@ class LGBMModel(_LGBMModelBase):
             Group data of training data.
         eval_set : list or None, optional (default=None)
             A list of (X, y) tuple pairs to use as a validation sets for early-stopping.
-        eval_names: list of strings or None, optional (default=None)
+        eval_names : list of strings or None, optional (default=None)
             Names of eval_set.
         eval_sample_weight : list of arrays or None, optional (default=None)
             Weights of eval data.
+        eval_class_weight : list or None, optional (default=None)
+            Class weights of eval data.
         eval_init_score : list of arrays or None, optional (default=None)
             Init score of eval data.
         eval_group : list of arrays or None, optional (default=None)
@@ -395,6 +399,7 @@ class LGBMModel(_LGBMModelBase):
             params['verbose'] = -1
         params.pop('silent', None)
         params.pop('n_estimators', None)
+        params.pop('class_weight', None)
         if self._n_classes is not None and self._n_classes > 2:
             params['num_class'] = self._n_classes
         if hasattr(self, '_eval_at'):
@@ -413,10 +418,16 @@ class LGBMModel(_LGBMModelBase):
             X, y = _LGBMCheckXY(X, y, accept_sparse=True, force_all_finite=False, ensure_min_samples=2)
             _LGBMCheckConsistentLength(X, y, sample_weight)
 
+        class_sample_weight = _LGBMComputeSampleWeight(self.class_weight, y)
+        if sample_weight is None or len(sample_weight) == 0:
+            sample_weight = class_sample_weight
+        else:
+            sample_weight = np.multiply(sample_weight, class_sample_weight)
+
         self._n_features = X.shape[1]
 
         def _construct_dataset(X, y, sample_weight, init_score, group, params):
-            ret = Dataset(X, label=y, max_bin=self.max_bin, weight=sample_weight, group=group, params=params)
+            ret = Dataset(X, label=y, weight=sample_weight, group=group, params=params)
             ret.set_init_score(init_score)
             return ret
 
@@ -439,8 +450,13 @@ class LGBMModel(_LGBMModelBase):
                         elif isinstance(collection, dict):
                             return collection.get(i, None)
                         else:
-                            raise TypeError('eval_sample_weight, eval_init_score, and eval_group should be dict or list')
+                            raise TypeError('eval_sample_weight, eval_class_weight, eval_init_score, and eval_group should be dict or list')
                     valid_weight = get_meta_data(eval_sample_weight, i)
+                    valid_class_sample_weight = _LGBMComputeSampleWeight(get_meta_data(eval_class_weight, i), valid_data[1])
+                    if valid_weight is None or len(valid_weight) == 0:
+                        valid_weight = valid_class_sample_weight
+                    else:
+                        valid_weight = np.multiply(valid_weight, valid_class_sample_weight)
                     valid_init_score = get_meta_data(eval_init_score, i)
                     valid_group = get_meta_data(eval_group, i)
                     valid_set = _construct_dataset(valid_data[0], valid_data[1], valid_weight, valid_init_score, valid_group, params)
@@ -578,16 +594,6 @@ class LGBMModel(_LGBMModelBase):
             raise LGBMNotFittedError('No feature_importances found. Need to call fit beforehand.')
         return self.booster_.feature_importance()
 
-    def booster(self):
-        warnings.warn('The `booster()` method is deprecated and will be removed in 2.0.12 version. '
-                      'Please use attribute `booster_` instead.', LGBMDeprecationWarning)
-        return self.booster_
-
-    def feature_importance(self):
-        warnings.warn('The `feature_importance()` method is deprecated and will be removed in 2.0.12 version. '
-                      'Please use attribute `feature_importances_` instead.', LGBMDeprecationWarning)
-        return self.feature_importances_
-
 
 class LGBMRegressor(LGBMModel, _LGBMRegressorBase):
     """LightGBM regressor."""
@@ -611,6 +617,9 @@ class LGBMRegressor(LGBMModel, _LGBMRegressorBase):
         return self
 
     base_doc = LGBMModel.fit.__doc__
+    fit.__doc__ = (base_doc[:base_doc.find('eval_class_weight :')] +
+                   base_doc[base_doc.find('eval_init_score :'):])
+    base_doc = fit.__doc__
     fit.__doc__ = (base_doc[:base_doc.find('eval_metric :')] +
                    'eval_metric : string, list of strings, callable or None, optional (default="l2")\n' +
                    base_doc[base_doc.find('            If string, it should be a built-in evaluation metric to use.'):])
@@ -622,7 +631,7 @@ class LGBMClassifier(LGBMModel, _LGBMClassifierBase):
     def fit(self, X, y,
             sample_weight=None, init_score=None,
             eval_set=None, eval_names=None, eval_sample_weight=None,
-            eval_init_score=None, eval_metric="logloss",
+            eval_class_weight=None, eval_init_score=None, eval_metric="logloss",
             early_stopping_rounds=None, verbose=True,
             feature_name='auto', categorical_feature='auto', callbacks=None):
         _LGBMCheckClassificationTargets(y)
@@ -658,6 +667,7 @@ class LGBMClassifier(LGBMModel, _LGBMClassifierBase):
                                         init_score=init_score, eval_set=eval_set,
                                         eval_names=eval_names,
                                         eval_sample_weight=eval_sample_weight,
+                                        eval_class_weight=eval_class_weight,
                                         eval_init_score=eval_init_score,
                                         eval_metric=eval_metric,
                                         early_stopping_rounds=early_stopping_rounds,
@@ -761,6 +771,9 @@ class LGBMRanker(LGBMModel):
         return self
 
     base_doc = LGBMModel.fit.__doc__
+    fit.__doc__ = (base_doc[:base_doc.find('eval_class_weight :')] +
+                   base_doc[base_doc.find('eval_init_score :'):])
+    base_doc = fit.__doc__
     fit.__doc__ = (base_doc[:base_doc.find('eval_metric :')] +
                    'eval_metric : string, list of strings, callable or None, optional (default="ndcg")\n' +
                    base_doc[base_doc.find('            If string, it should be a built-in evaluation metric to use.'):base_doc.find('early_stopping_rounds :')] +
